unit FoldPanel;
(***********************************************************************
 *                                                                     *
 *  Copyright (C) 1996-1997                                            *
 *  Erol S. Uzuner                                                     *
 *                                                                     *
 *  Modul       :       Foldpanel                                      *
 *  Version     :       1.0 (26.08.97)                                 *
 *                                                                     *
 *  Beschreibung:      TFoldPanel Komponente                            *
 *                                                                     *
 *  Autor       :       Erol S. Uzuner                                 *
 *                                                                     *
 *  Datum       :       Die Aug 29.08.97  23:37:00 1997                *
 *                                                                     *
 *                                                                     *
 *  Aenderungen :                                                      *
 *  Datum        Autor          Beschreibung                           *
 * 08.09.97     Erol S. Uzuner  TFoldPanelClient als ClientWindow       *
 *                              eingefhrt                             *
 * 09.09.97     Erol S. Uzuner  Die Komponente wurde auf den CLientWindow
 *                              angepasst.                             *
 * 11.09.97     Erol S. Uzuner  Jetzt auch lauffhig auf Delphi 2
 *                              Wenn DELPHI3 definiert ist wirds es
 *                              fr Delphi3 compiliert
 * 11.09.97     Erol S. Uzuner  FoldPosition eingefuegt.
 * 16.09.97     Erol S. Uzuner  Wenn Collapsed kann die Hoehe nicht veraendert
 *                              werden.
 * 16.01.99     Erol S. Uzuner  Das ClientWindows wurde zu einem
 *                              TPaneles umgewandelt.
 * 24.02.99     Erol S. Uzuner  Fehler beim auslsen von
 *                              On- und BevorExpand/Collapse beseitigt
 *
 ***********************************************************************)


 (**********************************************************************

     TFoldPanel  besteht aus einem TCustomPanel als Hauptcontainer und
     einem TCustomControl als TPaneles (ehemals ClientWindow), welches
     die auf dem FoldPanel plazierten Elemente beinhaltet.
     Das ClientWindow bekommt man durch die Funktion GetClientWindow.

  *********************************************************************)

interface

  {$I ES.inc}

uses
   Windows, Messages, Classes, ExtCtrls, Graphics, Controls, Forms, Paneles,
   BitmapHolder;

type

  TFoldStatus = (fsExpanded, fsCollapsed);
  TFoldPosition = (fpLeftTop, fpRightTop, fpLeftBottom, fpRightBottom);
  TFoldClickEvent = procedure( Sender:TObject; Status: TFoldStatus;
                               VAR DoFoldAction : Boolean) of object;

  TFoldPanelClient = class(TPaneles)
   protected
     procedure Paint; override;
   public
     MyDiff : Integer;
     FoldPosition : TFoldPosition;
     NoFolding : Boolean;
     constructor create(AOwner : TComponent); override;
     procedure setBounds(ALeft, ATop, AWidth, AHeight : Integer); override;
  end;

  TFoldPanel = class(TCustomPanel)
  private
      { Private-Deklarationen }
    FClientWindow : TFoldPanelClient;
    (* Status des Foldings, Expanded oder Collapsed *)
    FFoldStatus : TFoldStatus;
    (* Die hhe kann im collapsed zustand nicht verndert werden *)
    FHeightBlock : boolean;
    (* Glyphs zur Darstellung der Beiden Stati *)
    FCurrBmp,
    FExpandGlyph, FCollapsGlyph, FActiveExpandGlyph, FActiveCollapsGlyph  : TBitmap;
    (* Sollen die Bitmaps in den Stream geschrieben werden ? *)
    FStoreExpandGlyph, FStoreCollapsGlyph : Boolean;
    (* Active Color, wenn das Foldelement active ist *)
     FFoldActiveColor : TColor;
    (* Position des Fold Elements *)
    FFoldPosition : TFoldPosition;
    (* Wurde das Foldelement gedrckt *)
    FFoldBtnDown : Boolean;

    (* Dies ist die Trennlinie zwishen dem unterem Panel und dem
       ClientWindow *)
     FSeperated : Boolean;
     FLineStyle : TBevelStyle;

    (* Wird bentigt um den Mausclick abzufangen der auf den Folds
       gettigt wird, beinhaltet abwechselnd (je nach Status)
       den Rahmen der Glyphs *)
    FFoldRect : TRect;
    (* Um ein CLick ereignis auszulsen merken wir uns bei mouse down
       den Punkt und beim loslassen schauen wir ob dieser und der neue Punkt
       im FFoldRect liegen, wenn das der Fall ist, habn wir ein FoldClick
       Da Die Bitmaps nur gezeichnet werden und keine Ereignisse abfangen
       knnen, mssen wir uns auf diese weise helfen.
    *)
    FLastPoint : TPoint;
    (* Nach dem Collapsen muss die vorherige Gre hergestellt werden,
       deshalb merken wir uns die Gre hier und passen sie auch an bei einer
       grennderung durch den Benutzer *)
    FLastHeight : Integer;

    (* Beim gefalteten Zustand kann ein Text als Infiormation ntzlich sein *)
    FCaption : TCaption;
    FCaptionFont : TFont;

    (* Folding, Einschalten, Ausschalten des Foldings *)
    FFolding : Boolean;

    (* Controll ob es ein FoldClick war *)
    FIsFoldClick : Boolean;

    (*  Events zum Folding *)
    FOnFoldClick : TFoldClickEvent;
    FBeforeExpand,
    FAfterExpand,
    FBeforeCollaps,
    FAfterCollaps : TNotifyEvent;

    (* Property Proceduren *)
    function GetControl(Index: Integer): TControl;
    function GetControlCount: Integer;
    function GetComponent(AIndex: Integer): TComponent;
    function GetComponentCount: Integer;
    procedure setFoldStatus(value : TFoldStatus);
    function GetExpandGlyph: TBitmap;
    procedure SetExpandGlyph(Value: TBitmap);
    function GetCollapsGlyph: TBitmap;
    procedure SetCollapsGlyph(Value: TBitmap);
    procedure setInfoCaption(value : TCaption);
    procedure SetCaptionFont(Value: TFont);
    function StoreCaptionFont : Boolean;
    procedure SetFolding(Value : Boolean);
    function StoreExpandGlyph : Boolean;
    function StoreCollapsGlyph : Boolean;
    function GetBevelStyle(index : Integer) : TPanelBevel;
    procedure SetBevelStyle(index : Integer; Value : TPanelBevel);
    function GetBevelWidth(index : Integer) : TBevelWidth;
    procedure SetBevelWidth(index : Integer; Value : TBevelWidth);
    procedure SetLineStyle(Value: TBevelStyle);
    procedure SetSeperated(Value: Boolean);
    procedure SetFoldPosition(Value : TFoldPosition);
    procedure SetFoldActiveColor(Value : TColor);

    {surfacing events}
    procedure FClick(Sender : TObject);
    procedure FDblClick(Sender : TObject);
    procedure FDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure FDragOver(Sender, Source: TObject; X, Y: Integer;
                        State: TDragState; var Accept: Boolean);
    procedure FEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure FEnter(Sender : TObject);
    procedure FExit(Sender : TObject);
    procedure FKeyDown(Sender : TObject; var Key: Word; Shift: TShiftState);
    procedure FKeyPress(Sender : TObject; var Key: Char);
    procedure FKeyUp(Sender : TObject; var Key: Word; Shift: TShiftState);
    procedure FStartDrag(Sender : TObject; var DragObject: TDragObject);

    procedure FMouseDown(Sender: TObject; Button : TMouseButton;
                         Shift: TShiftState;X, Y: Integer);
    procedure FMouseUp(Sender: TObject; Button : TMouseButton;
                         Shift: TShiftState;X, Y: Integer);
    procedure FMouseMove(Sender: TObject; Shift: TShiftState;X, Y: Integer);

    {Procedures}
    function MakeLocalPoint(Control: TControl; X, Y: integer): TPoint;
    procedure DrawGlyph;
    procedure SetClientWindowBounds;

    (* Message handler *)
    procedure WMLButtonUp(var Message: TWMLButtonUp); message WM_LBUTTONUP;
    procedure WMLButtonDown(var Message: TWMLButtonDown); message WM_LBUTTONDOWN;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  {$IFDEF DELPHI3}
     procedure CMControlChange(var Message: TMessage); message CM_CONTROLCHANGE;
  {$ELSE}
    procedure CMControlChange(var Message: TMessage); message CM_CONTROLLISTCHANGE;
  {$ENDIF}

  protected
    { Protected-Deklarationen }
    procedure CreateActiveGlyph(source, dest : TBitmap);
    procedure Paint; override;
    (* Wir haben einen Click auf die Fold elemente entdeckt,
        also reagieren wir darauf *)
    procedure FoldClick; dynamic;
    (* Streaming der nicht Pulished Eigenschaften, des ClientPanels*)
   {$IFDEF DELPHI3}
     procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
   {$ELSE}
     procedure GetChildren(Proc: TGetChildProc); override;
   {$ENDIF}
    procedure Click; override;
    procedure SetChildOrder(Child: TComponent; Order: Integer); override;
    function GetBitmap : TBitmap;
    procedure SetBitmap(Value : TBitmap);
    function GetBitmapHolder : TBitmapHolder;
    procedure SetBitmapHolder(Value : TBitmapHolder);
  public
    { Public-Deklarationen }
    constructor create(AOwner : TComponent); override;
    destructor destroy; override;
    {Liefert das Clients fenster Zurck indem die Elemente Plaziert sind }
    function GetClientWindow : TCustomControl;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure setBounds(ALeft, ATop, AWidth, AHeight : Integer); override;

    {Liefert die Taborder}
    procedure GetTabOrderList(List: TList); override;

    function isCollapsed : Boolean;
    function isExpanded : Boolean;
    procedure Collaps;
    procedure Expand;

    {Controls auf dem Panel}
    property Controls[Index: Integer]: TControl read GetControl;
    property ControlCount: Integer read GetControlCount;
    property Components[Index: Integer]: TComponent read GetComponent;
    property ComponentCount: Integer read GetComponentCount;

  published
    { Published-Deklarationen }
    property Align;
    property BevelInner : TPanelBevel index 1 read GetBevelStyle write SetBevelStyle;
    property BevelOuter : TPanelBevel index 2 read GetBevelStyle write SetBevelStyle;
    property BevelWidth : TBevelWidth index 1 read GetBevelWidth write SetBevelWidth;
    property BorderWidth;
    property BorderStyle;
    property ClientBevelInner : TPanelBevel index 3 read GetBevelStyle write SetBevelStyle;
    property ClientBevelOuter : TPanelBevel index 4 read GetBevelStyle write SetBevelStyle;
    property ClientBevelWidth : TBevelWidth index 2 read GetBevelWidth write SetBevelWidth;
    property DragCursor;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property Color;
    property Ctl3D;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDrag;

    property FoldStatus : TFoldStatus read FFoldStatus write setFoldStatus default fsExpanded;
    property Folding : Boolean read FFolding write setFolding default true;
    property Caption : TCaption  read FCaption write setInfoCaption;
    property CaptionFont : TFont read FCaptionFont write SetCaptionFont stored StoreCaptionFont;
    property GlyphExpand : TBitmap read GetExpandGlyph write SetExpandGlyph stored StoreExpandGlyph;
    property GlyphCollaps : TBitmap read GetCollapsGlyph write SetCollapsGlyph stored StoreCollapsGlyph;
    property Seperated : Boolean read FSeperated write SetSeperated stored true default true;
    property SeperatorLineStyle : TBevelStyle read FLineStyle write SetLineStyle stored true default bsLowered;
    property FoldPosition : TFoldPosition read FFoldPosition write SetFoldPosition default fpLeftTop;
    property FoldActiveColor : TColor read FFoldActiveColor write SetFoldActiveColor default clAqua;

    {Events}
    property OnFoldClick: TFoldClickEvent read FOnFoldClick write FOnFoldClick;
    property BeforeExpand: TNotifyEvent read FBeforeExpand write FBeforeExpand;
    property AfterExpand: TNotifyEvent read FAfterExpand write FAfterExpand;
    property BeforeCollaps: TNotifyEvent read FBeforeCollaps write FBeforeCollaps;
    property AfterCollaps: TNotifyEvent read FAfterCollaps write FAfterCollaps;

    property Bitmap : TBitmap read GetBitmap write SetBitmap;
    property BitmapHolder : TBitmapHolder read GetBitmapHolder write SetBitmapHolder;
  end;


procedure Register;

implementation

  {$R *.RES}

procedure Register;
begin
  RegisterComponents('sarcon', [TFoldPanel]);
end;

function Rechteck(Left, Top, Right, Bottom : Integer) : TRect;
begin
    Result.Left := Left; Result.Top := Top;
    Result.Right := Right; Result.Bottom := Bottom;
end;

procedure TFoldPanelClient.setBounds(ALeft, ATop, AWidth, AHeight : Integer);
begin
   if NoFolding then
     inherited setBounds(ALeft, ATop, AWidth, AHeight)
   else begin
      case FoldPosition of
          fpLeftTop, fpLeftBottom :
                   inherited setBounds(MyDiff, ATop, AWidth - ABS(ALeft - MyDiff), AHeight);
          fpRightTop, fpRightBottom :
                   inherited setBounds(ALeft, ATop, AWidth - MyDiff, AHeight);
      end;
   end;
end;

constructor TFoldPanelClient.Create(AOwner : TComponent);
begin
    inherited Create(AOwner);
    ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents,
                     csSetCaption, csOpaque, csDoubleClicks, csReplicatable];
    MyDiff := 0;
    NoFolding := false;
    Align := alClient;
end;

constructor TFoldPanel.create(AOwner : TComponent);
begin
     inherited create(AOwner);
     FClientWindow := TFoldPanelClient.Create(self);
     with FClientWindow do
     begin
       Parent := self;
       OnClick := FCLick;
       OnDblCLick := FDblCLick;
       OnDragDrop := FDragDrop;
       OnDragOver := FDragOver;
       OnEndDrag :=  FEndDrag;
       OnEnter := FEnter;
       OnExit := FExit;
       OnKeyDown := FKeyDown;
       OnKeyUp := FKeyUp;
       OnKeyPress := FKeyPress;
       OnMouseDown := FMouseDown;
       OnMouseMove := FMouseMove;
       OnMouseUp := FMouseUp;
       OnStartDrag := FStartDrag;
       BevelInner := bvNone;
       BevelOuter := bvNone;
     end;

     ControlStyle := ControlStyle - [csSetCaption]; {csAcceptsControls];}
     FStoreExpandGlyph        := true;
     FStoreCollapsGlyph        := true;
     FExpandGlyph := TBitmap.Create;
     FCollapsGlyph := TBitmap.Create;
     FActiveExpandGlyph := TBitmap.Create;
     FActiveCollapsGlyph := TBitmap.Create;
     FCaptionFont := TFont.Create;
     {Wenn der FOnt verndert wird bitte auch  StoreCaptionFont anpassen}
     FCaptionFont.Name := 'Small Fonts';
     FCaptionFont.Size := 7;
     FLastHeight := height;
     FFolding := true;
     FSeperated := true;
     FFoldBtnDown := false;
     FIsFoldClick := false;
     FHeightBlock := false;
     FFoldActiveColor := clAqua;
     setExpandGlyph(nil);
     setCollapsGlyph(nil);
     FFoldStatus := fsCollapsed; {Damit auch setFoldStatus beim ersten mal funktioniert }
     setFoldStatus(fsExpanded);
end;

destructor TFoldPanel.destroy;
begin
    FClientWindow.Free;
    FExpandGlyph.Free;
    FCollapsGlyph.Free;
    FActiveExpandGlyph.Free;
    FActiveCollapsGlyph.Free;
    FCaptionFont.Free;
    inherited destroy;
end;

procedure TFoldPanelClient.Paint;
begin
    Canvas.Brush.Color := Color;
    Canvas.FillRect(ClientRect);
    inherited Paint;
end;

(***************************************************
 *
 *  Message Handler
 *
 ***************************************************)

procedure TFoldPanel.CMControlChange(var Message: TMessage);
var
  Control : TControl;
begin
   (* Ein untergeordnetes Element wird eingefgt,
      aufpassen das die Foldelemente nicht berlagert werden *)

   if Boolean(Message.lParam) then {InsertControl}
   begin
          Control := TControl(Message.wParam);
          if (Control <> FClientWindow) AND (Control.Parent <> FClientWindow) then
          begin
             Self.RemoveControl(Control);
             FClientWindow.InsertControl(Control);
          end;
   end else
      inherited;
end;

procedure TFoldPanel.WMLButtonDown(var Message: TWMLButtonDown);
var
   R : TRect;
begin
     FLastPoint := SmallPointToPoint(Message.Pos);
     if PtInRect(FFoldRect, FLastPoint) then
     begin
            R.left :=  FFoldRect.left + 1; R.Top := FFoldRect.Top + 1;
            R.right :=  FFoldRect.right - 1; R.bottom := FFoldRect.bottom - 1;
            FFoldBtnDown := true;
            Canvas.Pen.Color :=  clBtnShadow;
            Canvas.Rectangle(R.Left + 1, R.Top +1, R.Right, R.Bottom);
            Frame3D(Canvas, R, clBtnShadow, clBtnHighlight, 1);
     end;
     inherited;
end;

procedure TFoldPanel.WMLButtonUp(var Message: TWMLButtonUp);
begin
    if FFolding then
   begin
      if not (csDesigning in ComponentState) then
      begin
         (* Schauen wir mal ob wir ein Foldelement erwischt haben *)
          if PtInRect(FFoldRect, FLastPoint) AND
             PtInRect(FFoldRect, SmallPointToPoint(Message.Pos)) then
                     FIsFoldClick := true;

          if FFoldBtnDown then
          begin
             DrawGlyph;
             FFoldBtnDown := false;
          end;
      end;
   end;
   inherited;
end;

procedure TFoldPanel.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  doit : Boolean;
begin
   (* Zum enfrben der Foldelemente, aktivierungs anzeige und so weiter ...*)

  doit := true;
  if FFolding then
  begin
       if PtInRect(FFoldRect, Point(X, Y)) then
       begin
          case FFoldStatus of
            fsExpanded :  if FCurrBmp = FActiveCollapsGlyph then doit := false
                          else FCurrBmp := FActiveCollapsGlyph;
            fsCollapsed : if FCurrBmp = FActiveExpandGlyph then doit := false
                          else FCurrBmp := FActiveExpandGlyph;
          end;
       end else
       begin
           case FFoldStatus of
            fsExpanded :  if FCurrBmp = FCollapsGlyph then doit := false
                          else FCurrBmp := FCollapsGlyph;
            fsCollapsed : if FCurrBmp = FExpandGlyph then doit := false
                          else FCurrBmp := FExpandGlyph;
           end;

       end;
       if doit then
       DrawGlyph;
  end;

  inherited MouseMove(Shift, X, Y);

end;

procedure TFoldPanel.CMMouseLeave(var Message: TMessage);
var
  doit : Boolean;
begin
   doit := true;
   if FFolding then
   begin
     case FFoldStatus of
       fsExpanded :  if FCurrBmp = FCollapsGlyph then doit := false
                     else FCurrBmp := FCollapsGlyph;
       fsCollapsed : if FCurrBmp = FExpandGlyph then doit := false
                     else FCurrBmp := FExpandGlyph;
      end;

      if doit then DrawGlyph;
  end;
  inherited;
end;

procedure TFoldPanel.Paint;
begin

   inherited Paint;

   if FFolding then begin
     case FFoldStatus of
       fsExpanded : FCurrBmp := FCollapsGlyph;
       fsCollapsed : begin
            FCurrBmp := FExpandGlyph;
               if (BevelInner = bvNone) AND (BevelOuter = bvNone) then
               begin
                 Canvas.Pen.Color := clBtnShadow;
                 Canvas.Rectangle(-1, -1, Width, Height);
                 Canvas.Pen.Color := clBtnHighlight;
                 Canvas.Rectangle(0, 0, Width + 1, Height + 1);
               end;
         end;
     end;
     DrawGlyph;
   end;
end;

(***************************************************
 *
 *  Methods
 *
 ***************************************************)

function TFoldPanel.GetBitmap : TBitmap;
begin Result := FClientWindow.Bitmap;
end;

procedure TFoldPanel.SetBitmap(Value : TBitmap);
begin FClientWindow.Bitmap := Value;
end;

function TFoldPanel.GetBitmapHolder : TBitmapHolder;
begin Result := FClientWindow.BitmapHolder;
end;

procedure TFoldPanel.SetBitmapHolder(Value : TBitmapHolder);
begin FClientWindow.BitmapHolder := Value;
end;

procedure TFoldPanel.SetBounds(ALeft, ATop, AWidth, AHeight : Integer);
begin
   (* Wird die Gre gendert pasen wir auch FLastHeight an damit
     wir nach dem entfalten auch die richtige Gre herstellen *)
  if FFoldStatus = fsExpanded then FLastHeight :=  AHeight;
 {  if (csDesigning in ComponentState) then
  begin
       if AHeight < FCollapsGlyph.height then AHeight := FCollapsGlyph.height;
       if AWidth < FCollapsGlyph.width then AWidth := FCollapsGlyph.width;
  end;
  }
  if FHeightBlock then AHeight := Self.Height;

  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

{$IFDEF DELPHI3}
procedure TFoldPanel.GetChildren(Proc: TGetChildProc; Root: TComponent);
{$ELSE}
procedure TFoldPanel.GetChildren(Proc: TGetChildProc);
{$ENDIF}
var
  I: Integer;
  Control: TControl;
begin
  with FClientWindow do
  begin
     for I := 0 to ControlCount - 1 do
     begin
       Control := Controls[I];
       if Control.Parent = FClientWindow then Proc(Control);
     end;
  end;
end;

procedure TFoldPanel.SetChildOrder(Child: TComponent; Order: Integer);
begin
     FClientWindow.SetChildOrder(Child, Order);
     inherited SetChildOrder(Child, Order);
end;

procedure TFoldPanel.Click;
begin
    if FIsFoldClick then begin FoldClick;  FIsFoldClick := false; end
    else inherited Click;
end;
 
procedure TFoldPanel.FoldClick;
var
   DoAction : Boolean;
begin
     DoAction := True;
     if Assigned(FOnFoldClick) then OnFoldClick(self, FFoldStatus, DoAction);

     if DoAction then begin
        case FFoldStatus of
            fsExpanded :  setFoldStatus(fsCollapsed);
            fsCollapsed : setFoldStatus(fsExpanded);
        end;
     end;
end;

function TFoldPanel.GetClientWindow : TCustomControl;
begin Result := FClientWindow;
end;

procedure TFoldPanel.GetTabOrderList(List: TList);
begin FClientWindow.GetTabOrderList(List);
end;

function TFoldPanel.MakeLocalPoint(Control: TControl;
                                       X, Y: integer): TPoint;
{ this function takes a coordinate relative to some arbitrary }
{ control and returns a coordinate relative to self }
begin
  Result.X := X;
  Result.Y := Y;
  Result := ScreenToClient(Control.ClientToScreen(Result));
end;

procedure TFoldPanel.SetClientWindowBounds;
var
   rect : TRect;
begin
     if FCurrBmp <> Nil then
        FClientWindow.MyDiff := FCurrBmp.Width
     else
        FClientWindow.MyDiff :=  0;
     if BevelInner <> bvNone then Inc(FClientWindow.MyDiff, BevelWidth);
     if BevelOuter <> bvNone then Inc(FClientWindow.MyDiff, BevelWidth);
     if FFolding and FSeperated then Inc(FClientWindow.MyDiff, 2);

     rect := Bounds(0, 0, Width, Height);
     AlignControls(FClientWindow, rect);
end;

procedure TFoldPanel.DrawGlyph;
var
   dy, th, cl : Integer;
   r, d : TRect;

   procedure IncRect(VAR rect : TRect; n : Integer);
   begin
       case FFoldPosition of
          fpLeftTop :
             begin
                  INC(rect.Left, n);  INC(rect.Top, n);
                  INC(rect.Right, n); INC(rect.Bottom, n);
             end;
          fpLeftBottom :
             begin
                  INC(rect.Left, n);  DEC(rect.Top, n);
                  INC(rect.Right, n); DEC(rect.Bottom, n);
             end;
          fpRightTop:
             begin
                  DEC(rect.Left, n);  INC(rect.Top, n);
                  DEC(rect.Right, n); INC(rect.Bottom, n);
             end;

          fpRightBottom :
             begin
                  DEC(rect.Left, n);  DEC(rect.Top, n);
                  DEC(rect.Right, n); DEC(rect.Bottom, n);
             end;
       end;
   end;

   procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
   begin
     with Canvas do
     begin
       Pen.Color := C;
       MoveTo(X1, Y1);
       LineTo(X2, Y2);
     end;
   end;

   procedure Seperator;
   var
      Color1, Color2: TColor;
      l : TRect;
   begin
       with Canvas do
       begin
          if FLineStyle = bsLowered then
          begin
            Color1 := clBtnShadow;
            Color2 := clBtnHighlight;
          end
          else
          begin
            Color1 := clBtnHighlight;
            Color2 := clBtnShadow;
          end;

          case FFoldPosition of
            fpLeftTop:     l := Rechteck( d.right, d.top, d.right, Height - d.top);
            fpLeftBottom : l := Rechteck( d.right, Height - d.Bottom, d.right, d.bottom);
            fpRightTop:    l := Rechteck( d.left, d.top, d.left, Height - d.top);
            fpRightBottom: l := Rechteck( d.left, Height - d.Bottom, d.left, d.bottom);
          end;

          BevelLine(Color1, l.left, l.top, l.right, l.bottom);
          BevelLine(Color2, l.left+1, l.top, l.right+1, l.bottom);
       end;
   end;

begin
   r.left := 0; r.top := 0; r.bottom := FCurrBmp.height; r.right := FCurrBmp.width;

     case FFoldPosition of
          fpLeftTop :
              begin
                   d.left := r.left;
                   d.top := r.top;
                   d.bottom := r.bottom;
                   d.right := r.right;
              end;
          fpLeftBottom :
              begin
                   d.left := r.left;
                   d.top := Self.Height - (r.top + r.Bottom);
                   d.bottom := d.top + r.bottom;
                   d.right := r.right;
              end;
          fpRightTop:
              begin
                  d.left := Self.Width - (r.left + r.right);
                  d.top := r.top;
                  d.right := d.left + r.right;
                  d.bottom := r.bottom;
              end;
          fpRightBottom :
              begin
                  d.left := Self.Width - (r.left + r.right);
                  d.top := Self.Height - (r.top + r.Bottom);
                  d.right := d.left + r.right;
                  d.bottom := d.top + r.bottom;
              end;
       end;


   {IncRect Arbeitet je nach FoldPosition, bei linke Seite wird addiert,
    bei der Rechten Seite subtrahiert}
   if BevelInner <> bvNone then IncRect(d, BevelWidth);
   if BevelOuter <> bvNone then IncRect(d, BevelWidth);

   Canvas.BrushCopy(d, FCurrBmp, r, FCurrBmp.TransparentColor);

   {Zeichne trennlinie}
    if FFolding AND FSeperated then Seperator;

   {Setze FoldRect }
   FFoldRect := d;

   if (FFoldStatus = fsCollapsed) AND (FCaption <> '') then
   begin
        Canvas.Font.Assign(FCaptionFont);
        th := Canvas.TextHeight(FCaption);
        if  th > d.bottom then  dy := 0
        else dy := d.top + (ABS(d.bottom - th) DIV 2);

        cl := 0;
        case FFoldPosition of
           fpLeftTop, fpLeftBottom : cl :=  d.right + 10;
           fpRightTop, fpRightBottom : cl := d.left - (Canvas.TextWidth(FCaption) + 10);
        end;
        Canvas.TextOut(cl, dy, FCaption);
   end;
end;

(***************************************************
 *
 *  Property Methods
 *
 ***************************************************)

function TFoldPanel.GetControl(Index: Integer): TControl;
begin Result := FClientWindow.Controls[Index];
end;

function TFoldPanel.GetControlCount: Integer;
begin Result := FClientWindow.ControlCount;
end;

function TFoldPanel.GetComponent(AIndex: Integer): TComponent;
begin Result := FClientWindow.Components[AIndex];
end;

function TFoldPanel.GetComponentCount: Integer;
begin Result := FClientWindow.ComponentCount;
end;

function TFoldPanel.GetBevelStyle(index : Integer) : TPanelBevel;
begin
    Result := bvNone;
    case index of
       1: Result := inherited BevelInner;
       2: Result := inherited BevelOuter;
       3 : Result := FClientWindow.BevelInner;
       4 : Result := FClientWindow.BevelOuter;
    end;
end;

procedure TFoldPanel.SetBevelStyle(index : Integer; Value : TPanelBevel);
begin
    case index of
       1: begin
              if inherited BevelInner <> Value then
              begin
                  inherited BevelInner := Value;
                  SetClientWindowBounds;
              end;
          end;
       2: begin
               inherited BevelOuter := Value;
               SetClientWindowBounds;
          end;
       3 : begin
                if FClientWindow.BevelInner <> Value then
                   FClientWindow.BevelInner := Value;
           end;
       4 : begin
                if FClientWindow.BevelOuter <> Value then
                   FClientWindow.BevelOuter := Value;
           end;
    end;
end;

function TFoldPanel.GetBevelWidth(index : Integer) : TBevelWidth;
begin
  case index of
   1 : Result := inherited BevelWidth;
   2 : Result := FClientWindow.BevelWidth;
   else Result := inherited BevelWidth;
  end;
end;

procedure TFoldPanel.SetBevelWidth(index : Integer; Value : TBevelWidth);
begin
   case index of
     1: begin
           if inherited BevelWidth <> Value then
           begin
               inherited BevelWidth := Value;
               SetClientWindowBounds;
           end;
        end;
     2 : begin
           if FClientWindow.BevelWidth <> Value then
               FClientWindow.BevelWidth := Value;
         end;
   end;
end;

procedure TFoldPanel.SetLineStyle(Value: TBevelStyle);
begin
     if FLineStyle <> Value then
     begin
         FLineStyle := Value;
         if FFolding then Invalidate;
     end;
end;

procedure TFoldPanel.SetSeperated(Value: Boolean);
begin
     if FSeperated <> Value then
     begin
         FSeperated := Value;
         if FFolding then Invalidate;
     end;
end;

procedure TFoldPanel.SetFoldPosition(Value : TFoldPosition);
var
   rect : TRect;
begin
    if FFoldPosition <> Value then
    begin
        FClientWindow.FoldPosition := Value;

        FFoldPosition := Value;
        if FFolding then
        begin
           rect := Bounds(0, 0, Width, Height);
           AlignControls(FClientWindow, rect);
           Invalidate;
        end;
    end;
end;

procedure TFoldPanel.SetFoldActiveColor(Value : TColor);
begin
   if FFoldActiveColor <> Value then
   begin
     FFoldActiveColor := value;
     CreateActiveGlyph(FExpandGlyph, FActiveExpandGlyph);
     CreateActiveGlyph(FCollapsGlyph, FActiveCollapsGlyph);
    end;
end;

procedure TFoldPanel.CreateActiveGlyph(source, dest : TBitmap);
var
   r : TRect;
   BlueScreen : TBitmap;
begin

   dest.Assign(source);
   r := Rechteck(0, 0, dest.Width, dest.Height);

   BlueScreen := TBitmap.Create;
   with BlueScreen DO begin
        Width := r.right;
        Height := r.bottom;
        Canvas.Brush.Color := FFoldActiveColor;
        Canvas.FillRect(r);
   end;

   WITH dest.Canvas DO BEGIN
     copymode := cmSrcAnd;
     CopyRect(r, BlueScreen.Canvas,r);
   END;
   BlueScreen.Free;
end;

function TFoldPanel.GetExpandGlyph: TBitmap;
begin
  Result := FExpandGlyph;
end;

procedure TFoldPanel.SetExpandGlyph(Value: TBitmap);

begin
   if Assigned(value) then
   begin
     FExpandGlyph.Assign(Value);
     FStoreExpandGlyph := true;
  end else begin
     FExpandGlyph.LoadFromResourceName(HInstance, 'EXPAND');
     FStoreExpandGlyph := false;
  end;

  CreateActiveGlyph(FExpandGlyph, FActiveExpandGlyph);

   SetClientWindowBounds;
   Invalidate;
end;



function TFoldPanel.GetCollapsGlyph: TBitmap;
begin
  Result := FCollapsGlyph;
end;

procedure TFoldPanel.SetCollapsGlyph(Value: TBitmap);
begin
  if Assigned(value) then
  begin
     FCollapsGlyph.Assign(Value);
     FStoreCollapsGlyph := true;

 end else begin
     FCollapsGlyph.LoadFromResourceName(HInstance, 'COLLAPS');
     FStoreCollapsGlyph := false;
  end;

  CreateActiveGlyph(FCollapsGlyph, FActiveCollapsGlyph);

  SetClientWindowBounds;
  Invalidate;
end;

procedure TFoldPanel.SetFolding(Value: Boolean);
begin
  if FFolding <> Value then
  begin
       FFolding := Value;
       if NoT(value) then
       begin
        FClientWindow.NoFolding := true;
        if (FFoldStatus = fsCollapsed) then
          setFoldStatus(fsExpanded)
       end else begin
          FClientWindow.NoFolding := false;
          Invalidate;
       end;
  end;
end;

procedure TFoldPanel.setFoldStatus(value : TFoldStatus);
var  h : Integer;
begin
   if (NoT(FFolding) OR (csDesigning in ComponentState))
      AND (value = fsCollapsed) then Exit;

   FCurrBmp := nil;
   if value <>  FFoldStatus then begin
     case value of
       fsExpanded : begin
           if Assigned(FBeforeExpand) then FBeforeExpand(self);
           FFoldStatus := value;
           FHeightBlock := false;
           FCurrBmp := FCollapsGlyph;
           case FFoldPosition of
             fpLeftTop, fpRightTop :
                SetBounds(Left, Top, Width, FLastHeight);
             fpLeftBottom, fpRightBottom :
                SetBounds(Left, Top - (FLastHeight - Height), Width, FLastHeight);
           end;
           SetClientWindowBounds;
           FClientWindow.Visible := true;
          end;
       fsCollapsed : begin
          if Assigned(FBeforeCollaps) then FBeforeCollaps(self);
          FFoldStatus := value;
          FClientWindow.Visible := False;
          FCurrBmp := FExpandGlyph;
          h := FCurrBmp.height;
          if BorderStyle = bsSingle then Inc(h, 2);
          if BevelInner <> bvNone   then Inc(h, BevelWidth * 2);
          if BevelOuter <> bvNone   then Inc(h, BevelWidth * 2);
          case FFoldPosition of
             fpLeftTop, fpRightTop :
                SetBounds(Left, Top, Width, h);
             fpLeftBottom, fpRightBottom :
                SetBounds(Left, (Top + Height) - h, Width, h);
          end;
          FHeightBlock := true;
        end;
     end;
     FFoldRect := Bounds(0, 0, FCurrBmp.width, FCurrBmp.height);
     invalidate;

     case value of
       fsExpanded  : if Assigned(FAfterExpand) then AfterExpand(self);
       fsCollapsed : if Assigned(FAfterCollaps) then AfterCollaps(self);
     end;
   end;
end;

procedure TFoldPanel.setInfoCaption(value : TCaption);
begin
     if Value <> FCaption then
     begin
         FCaption := Value;
          if not (csDesigning in ComponentState) then invalidate;
     end;
end;

procedure TFoldPanel.SetCaptionFont(Value: TFont);
begin
    FCaptionFont.Assign(Value);
end;

function  TFoldPanel.StoreCaptionFont : Boolean;
var
   tmpf : tFont;
begin
     tmpf := TFont.Create;
     tmpf.Name := 'Small Fonts';
     tmpf.Size := 7;

     Result := {$IFDEF DELPHI3} (FCaptionFont.Charset = tmpf.Charset) AND {$ENDIF}
               (FCaptionFont.Color  = tmpf.Color) AND
               (FCaptionFont.Height = tmpf.Height) AND
               (FCaptionFont.Name   = tmpf.Name) AND
               (FCaptionFont.Pitch  = tmpf.Pitch) AND
               (FCaptionFont.Size   = tmpf.Size) AND
               (FCaptionFont.Style  = tmpf.Style);
     Result := Not(Result);
end;

function TFoldPanel.StoreExpandGlyph : Boolean;
begin Result := FStoreExpandGlyph;
end;

function TFoldPanel.StoreCollapsGlyph : Boolean;
begin  Result := FStoreCollapsGlyph;
end;

{Surfaced Events}
procedure TFoldPanel.FClick(Sender: TObject);
begin
 if Assigned(OnClick) then OnClick(Self);
end;

procedure TFoldPanel.FDblClick(Sender: TObject);
begin
 if Assigned(OnDblClick) then OnDblClick(Self);
end;

procedure TFoldPanel.FDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  Pt: TPoint;
begin
  Pt := MakeLocalPoint(Sender as TControl, X, Y);
  if Assigned(OnDragDrop) then OnDragDrop(Self, Source, Pt.X, Pt.Y);
end;

procedure TFoldPanel.FDragOver(Sender, Source: TObject; X, Y: Integer;
                         State: TDragState; var Accept: Boolean);
var
  Pt: TPoint;
begin
  Pt := MakeLocalPoint(Sender as TControl, X, Y);
  if Assigned(OnDragOver) then OnDragOver(Self, Source, Pt.X, Pt.Y, State, Accept);
end;

procedure TFoldPanel.FEndDrag(Sender, Target: TObject; X, Y: Integer);
var
  Pt: TPoint;
begin
  Pt := MakeLocalPoint(Sender as TControl, X, Y);
  if Assigned(OnEndDrag) then OnEndDrag(Self, Target, Pt.X, Pt.Y);
end;

procedure TFoldPanel.FEnter(Sender : TObject);
begin
      if Assigned(OnEnter) then OnEnter(Self);
end;

procedure TFoldPanel.FExit(Sender : TObject);
begin
      if Assigned(OnExit) then OnExit(Self);
end;

procedure TFoldPanel.FKeyDown(Sender : TObject; var Key: Word; Shift: TShiftState);
begin
     if Assigned(OnKeyDown) then OnKeyDown(Self, Key, Shift);
end;
procedure TFoldPanel.FKeyPress(Sender : TObject; var Key: Char);
begin
   if Assigned(OnKeyPress) then OnKeyPress(Self, Key);
end;

procedure TFoldPanel.FKeyUp(Sender : TObject; var Key: Word; Shift: TShiftState);
begin
     if Assigned(OnKeyUp) then OnKeyUp(Self, Key, Shift);
end;

procedure TFoldPanel.FMouseDown(Sender: TObject; Button: TMouseButton;
                                    Shift: TShiftState; X, Y: Integer);
{ OnMouseDown handler  Resurfaces event. }
var
  Pt: TPoint;
begin
  Pt := MakeLocalPoint(Sender as TControl, X, Y);
  if Assigned(OnMouseDown) then OnMouseDown(Self, Button, Shift, Pt.X, Pt.Y);
end;

procedure TFoldPanel.FMouseUp(Sender: TObject; Button: TMouseButton; Shift:
                                 TShiftState; X, Y: Integer);
{ OnMouseUp handler.  Resurfaces event. }
var
  Pt: TPoint;
begin
  Pt := MakeLocalPoint(Sender as TControl, X, Y);
  if Assigned(OnMouseUp) then OnMouseUp(Self, Button, Shift, Pt.X, Pt.Y);
end;

procedure TFoldPanel.FMouseMove(Sender: TObject; Shift: TShiftState;
                                   X, Y: Integer);
{ OnMouseMove handler,  Resurfaces event. }
var
  Pt: TPoint;
begin
  Pt := MakeLocalPoint(Sender as TControl, X, Y);
  if Assigned(OnMouseMove) then OnMouseMove(Self, Shift, Pt.X, Pt.Y);
end;

procedure TFoldPanel.FStartDrag(Sender : TObject; var DragObject: TDragObject);
begin
   if Assigned(OnStartDrag) then OnStartDrag(Self, DragObject);
end;


function TFoldPanel.isCollapsed : Boolean;
begin Result := FFoldStatus = fsCollapsed;
end;

function TFoldPanel.isExpanded : Boolean;
begin Result := FFoldStatus = fsExpanded;
end;

procedure TFoldPanel.Collaps;
begin  setFoldStatus(fsCollapsed);
end;

procedure TFoldPanel.Expand;
begin setFoldStatus(fsExpanded);
end;

end.
